import {Component, ElementRef, HostBinding, Inject, Input, OnDestroy, Optional, Self, ViewChild} from '@angular/core';
import {SaveRegularObservationsRequestBloodPressure} from '@nspop/gm-web-facade-api';
import {AbstractControl, ControlValueAccessor, FormBuilder, FormGroup, NgControl} from '@angular/forms';
import {MAT_FORM_FIELD, MatFormField, MatFormFieldControl} from '@angular/material/form-field';
import {Subject} from 'rxjs';
import {FocusMonitor} from '@angular/cdk/a11y';
import {BloodPressureValidator} from './blood-pressure-input.validation';

@Component({
  selector: 'app-blood-pressure-input',
  templateUrl: './blood-pressure-input.component.html',
  styleUrls: ['./blood-pressure-input.component.scss'],
  providers: [{provide: MatFormFieldControl, useExisting: BloodPressureInputComponent}],
})
export class BloodPressureInputComponent implements
  ControlValueAccessor,
  MatFormFieldControl<SaveRegularObservationsRequestBloodPressure>,
  OnDestroy {

  public static nextId = 0;
  @ViewChild('systolic') systolicInput: HTMLInputElement | undefined;
  @ViewChild('diastolic') diastolicInput: HTMLInputElement | undefined;

  constructor(
    fb: FormBuilder,
    private focusMonitor: FocusMonitor,
    private elementRef: ElementRef<HTMLElement>,
    @Optional() @Inject(MAT_FORM_FIELD) public formField: MatFormField,
    @Optional() @Self() public ngControl: NgControl
  ) {
    this.parts =  fb.group({
      systolic: undefined,
      diastolic: undefined
    }, {
      validators: [BloodPressureValidator.ifOneFieldFilledOtherShouldBeFilledValidator('systolic', 'diastolic')]
    });

    if (this.ngControl != null) {
      this.ngControl.valueAccessor = this;
    }
  }

  @HostBinding() public id = `app-blood-pressure-input-${BloodPressureInputComponent.nextId++}`;
  public readonly controlType: string = 'blood-pressure-input';

  @Input('aria-describedby') userAriaDescribedBy: string = '';
  @Input() placeholder: string = '';
  @Input() required: boolean = false;
  @Input() label: string = 'Blodtryk';

  public parts: FormGroup;
  public stateChanges = new Subject<void>();
  public focused: boolean = false;
  public touched: boolean = false;
  public autofilled: boolean = false;
  public invalid: boolean = false;

  private innerDisabled = false;

  public onChangeCallback: (newValue: SaveRegularObservationsRequestBloodPressure | null) => any = () => {};
  public onTouchedCallback: () => any = () => {};

  @Input()
  get disabled(): boolean { return this.innerDisabled; }
  set disabled(value: boolean) {
    this.innerDisabled = value;
    this.innerDisabled ? this.parts.disable() : this.parts.enable();
    this.stateChanges.next();
  }

  @Input()
  get value(): SaveRegularObservationsRequestBloodPressure | null {
    const {systolic, diastolic} = this.parts.value;
    if (systolic && diastolic && systolic.length !== 0 && diastolic.length !== 0) {
      return { systolic, diastolic };
    }
    return null;
  }
  set value(bloodPressure: SaveRegularObservationsRequestBloodPressure | null) {
    const { systolic, diastolic } = bloodPressure || {};
    this.parts.setValue({ systolic: systolic || null, diastolic: diastolic || null});
    this.stateChanges.next();
  }

  get empty(): boolean {
    const {systolic, diastolic} = this.parts.value;
    return !systolic && !diastolic;
  }

  @HostBinding('class.floating')
  get shouldLabelFloat(): boolean {
    return this.focused || !this.empty;
  }

  public ngOnDestroy(): void {
    this.stateChanges.complete();
  }

  public setDescribedByIds(ids: string[]): void {
    const controlElement = this.elementRef
      .nativeElement
      .querySelector('.example-tel-input-container');
    controlElement?.setAttribute('aria-describedby', ids.join(' '));
  }

  onContainerClick(event: MouseEvent): void {
    if ((event.target as Element).tagName.toLowerCase() !== 'input') {
      this.elementRef?.nativeElement?.querySelector('input')?.focus();
    }
  }

  onFocusIn(event: FocusEvent): void {
    if (!this.focused) {
      this.focused = true;
      this.stateChanges.next();
    }
  }

  onFocusOut(event: FocusEvent): void {
    if (!this.elementRef.nativeElement.contains(event.relatedTarget as Element)) {
      this.touched = true;
      this.focused = false;
      this.onTouchedCallback();
      this.stateChanges.next();
    }
  }

  public handleInput(control: AbstractControl, nextElement?: HTMLInputElement): void {
    this.autoFocusNext(control, nextElement);
    this.onChangeCallback(this.value);
  }

  public handleRightInputNavigation(nextElement: HTMLInputElement, inputElement: HTMLInputElement): void {
    const cursorPos = inputElement.selectionStart;
    if (cursorPos && cursorPos === inputElement.value.length) {
      this.focusMonitor.focusVia(nextElement, 'program');
    }
  }

  public handleLeftInputNavigation(prevElement: HTMLInputElement, inputElement: HTMLInputElement): void {
    const cursorPos = inputElement.selectionStart;
    if (cursorPos === 0) {
      this.focusMonitor.focusVia(prevElement, 'program');
    }
  }

  public autoFocusNext(control: AbstractControl, nextElement?: HTMLInputElement): void {
    if (!control.errors && control.value.length === 3 && nextElement) {
      this.focusMonitor.focusVia(nextElement, 'program');
    }
  }

  public autoFocusPrev(control: AbstractControl, prevElement: HTMLInputElement): void {
    if (control.value.length < 1) {
      this.focusMonitor.focusVia(prevElement, 'program');
    }
  }

  get errorState(): boolean {
    const onlyOneIsFilled = this.isSystolicFilled() !== this.isDiastolicFilled();
    return this.touched && onlyOneIsFilled;
  }

  private isSystolicFilled(): boolean {
    const { systolic } = this.parts.value;
    return systolic && systolic.length > 0;
  }

  private isDiastolicFilled(): boolean {
    const { diastolic } = this.parts.value;
    return diastolic && diastolic.length > 0;
  }

  registerOnChange(fn: any): void {
    this.onChangeCallback = fn;
  }
  registerOnTouched(fn: any): void {
    this.onTouchedCallback = fn;
  }

  writeValue(bloodPressure: SaveRegularObservationsRequestBloodPressure | undefined): void {
    this.value = bloodPressure || null;
  }

}
