import {Component, Input, OnInit} from '@angular/core';
import {ChartDataSets, ChartOptions, ChartTooltipModel} from 'chart.js';
import { Color } from 'ng2-charts';
import {Observable} from 'rxjs';
import {ObservationTable, ObservationTableMotherRows} from '@nspop/gm-web-facade-api';

@Component({
  selector: 'app-line-chart',
  templateUrl: './line-chart.component.html',
  styleUrls: ['./line-chart.component.scss']
})
export class LineChartComponent implements OnInit {
  constructor() { }
  @Input() public data: Observable<ObservationTable | undefined> | undefined;
  public lineChartData: ChartDataSets[] = [];
  public lineChartOptions: ChartOptions = {
    responsive: true,
    maintainAspectRatio: false,
    layout: {
      padding: {
        left: 50,
        right: 50,
        top: 20,
        bottom: 20
      }
    },
    tooltips: {
      custom: (tooltipModel: ChartTooltipModel) => {
        if (tooltipModel.dataPoints) {
          tooltipModel.height = 75;
          tooltipModel.width = 225;
          // oh god. _titleAlign adds an annoying delay before aligning title to center and titleAlign is no longer available
          tooltipModel.title = ['                 Måling'];
          tooltipModel.titleSpacing = 20;
          tooltipModel.bodySpacing = 10;
          tooltipModel.backgroundColor = '#4228C4';
          tooltipModel.bodyFontColor = '#FFF';
          tooltipModel.titleFontColor = '#FFF';
          tooltipModel.cornerRadius = 20;
          tooltipModel.titleFontSize = 16;
          tooltipModel.bodyFontSize = 14;
          tooltipModel.caretSize = 0;
          let gestationAge = tooltipModel.dataPoints[0].xLabel;
          if (gestationAge && typeof gestationAge === 'string') {
              gestationAge = Number.parseInt(gestationAge, 10);
          }
          gestationAge = LineChartComponent.getGestationAge(gestationAge as number);
          tooltipModel.body = [
            {before: ['Symfyse-Fundus mål: ' + tooltipModel.dataPoints[0].yLabel + ' cm', 'Gestationsalder: ' +
              gestationAge], lines: [], after: []}
          ];
        }
      }
    },
    legend: {display: false},
    scales: {
      yAxes: [
        {
          ticks: {
            beginAtZero: true,
            max: 40,
            stepSize: 10,
          }
        }
      ],
      xAxes: [
        {
          ticks: {
            beginAtZero: true,
            suggestedMax: 40
          }
        }
      ]
    }
  };
  public lineChartColors: Color[] = [
    {
      borderColor: 'black',
      backgroundColor: '#FFF',
    },
  ];
  public lineChartLegend = true;
  public lineChartPlugins = [];

  private static getGestationAge(gestationAge: number): string {
    const week = Math.floor(gestationAge);
    const decimal = gestationAge % 1;
    const dayOfWeek = decimal / (1 / 7);
    const gestationDay = dayOfWeek >= 1 ? dayOfWeek - 1 : dayOfWeek;
    return [week, '+', Math.floor(gestationDay)].join('');
  }

  ngOnInit(): void {
    if (this.data) {
      this.data.subscribe( data => {
        if (data?.mother) {
          const motherRows = data.mother.rows;
          if (motherRows) {
            const rowData = this.getMotherRows(motherRows);
            this.lineChartData = [
              {
                label: 'Symfyse-fundusmål',
                data: rowData,
                pointRadius: 6,
                pointBackgroundColor: '#4228C4',
                pointBorderColor: '#FFF',
                borderWidth: 2
              }
            ];
          }
        }
      });
    }
  }

  public getMotherRows(motherRows: ObservationTableMotherRows[]): { x: number | undefined; y: number | undefined }[] {
    return motherRows
      .filter((row)  => row && row.pubismeasurement?.value && row.weeksIntoPregnancy)
      .map(row => {
        return { y: row?.pubismeasurement?.value, x: row.weeksIntoPregnancy };
    });
  }
}
